@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2012, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -12,8 +12,12 @@
 # The above copyright notice and this permission notice shall be
 # included in all copies or substantial portions of the Software.
 
-__version__ = '1.0.0'
+__version__ = '1.2.1'
 __author__ = 'Adrian Sampson <adrian@radbox.org>'
 
 import beets.library
+from beets.util import confit
+
 Library = beets.library.Library
+
+config = confit.LazyConfig('beets', __name__)
