@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2012, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -16,11 +16,13 @@
 """
 import logging
 import musicbrainzngs
+import re
 import traceback
 
 import beets.autotag.hooks
 import beets
 from beets import util
+from beets import config
 
 SEARCH_LIMIT = 5
 VARIOUS_ARTISTS_ID = '89ad4ac3-39f7-470e-963a-56509c546377'
@@ -44,18 +46,46 @@ class MusicBrainzAPIError(util.HumanReadableException):
 log = logging.getLogger('beets')
 
 RELEASE_INCLUDES = ['artists', 'media', 'recordings', 'release-groups',
-                    'labels', 'artist-credits']
-TRACK_INCLUDES = ['artists']
+                    'labels', 'artist-credits', 'aliases']
+TRACK_INCLUDES = ['artists', 'aliases']
 
-# python-musicbrainz-ngs search functions: tolerate different API versions.
-if hasattr(musicbrainzngs, 'release_search'):
-    # Old API names.
-    _mb_release_search = musicbrainzngs.release_search
-    _mb_recording_search = musicbrainzngs.recording_search
-else:
-    # New API names.
-    _mb_release_search = musicbrainzngs.search_releases
-    _mb_recording_search = musicbrainzngs.search_recordings
+def configure():
+    """Set up the python-musicbrainz-ngs module according to settings
+    from the beets configuration. This should be called at startup.
+    """
+    musicbrainzngs.set_hostname(config['musicbrainz']['host'].get(unicode))
+    musicbrainzngs.set_rate_limit(
+        config['musicbrainz']['ratelimit_interval'].as_number(),
+        config['musicbrainz']['ratelimit'].get(int),
+    )
+
+def _preferred_alias(aliases):
+    """Given an list of alias structures for an artist credit, select
+    and return the user's preferred alias alias or None if no matching
+    alias is found.
+    """
+    if not aliases:
+        return
+
+    # Only consider aliases that have locales set.
+    aliases = [a for a in aliases if 'locale' in a]
+
+    # Search configured locales in order.
+    for locale in config['import']['languages'].as_str_seq():
+        # Find matching aliases for this locale.
+        matches = [a for a in aliases if a['locale'] == locale]
+        # Skip to the next locale if we have no matches
+        if not matches:
+            continue
+
+        # Find the aliases that have the primary flag set.
+        primaries = [a for a in matches if 'primary' in a]
+        # Take the primary if we have it, otherwise take the first
+        # match with the correct locale.
+        if primaries:
+            return primaries[0]
+        else:
+            return matches[0]
 
 def _flatten_artist_credit(credit):
     """Given a list representing an ``artist-credit`` block, flatten the
@@ -73,12 +103,19 @@ def _flatten_artist_credit(credit):
             artist_sort_parts.append(el)
 
         else:
+            alias = _preferred_alias(el['artist'].get('alias-list', ()))
+
             # An artist.
-            cur_artist_name = el['artist']['name']
+            if alias:
+                cur_artist_name = alias['alias']
+            else:
+                cur_artist_name = el['artist']['name']
             artist_parts.append(cur_artist_name)
 
             # Artist sort name.
-            if 'sort-name' in el['artist']:
+            if alias:
+                artist_sort_parts.append(alias['sort-name'])
+            elif 'sort-name' in el['artist']:
                 artist_sort_parts.append(el['artist']['sort-name'])
             else:
                 artist_sort_parts.append(cur_artist_name)
@@ -124,9 +161,10 @@ def track_info(recording, index=None, medium=None, medium_index=None):
     info.decode()
     return info
 
-def _set_date_str(info, date_str):
+def _set_date_str(info, date_str, original=False):
     """Given a (possibly partial) YYYY-MM-DD string and an AlbumInfo
-    object, set the object's release date fields appropriately.
+    object, set the object's release date fields appropriately. If
+    `original`, then set the original_year, etc., fields.
     """
     if date_str:
         date_parts = date_str.split('-')
@@ -137,6 +175,9 @@ def _set_date_str(info, date_str):
                     date_num = int(date_part)
                 except ValueError:
                     continue
+
+                if original:
+                    key = 'original_' + key
                 setattr(info, key, date_num)
 
 def album_info(release):
@@ -159,7 +200,7 @@ def album_info(release):
                             int(medium['position']),
                             int(track['position']))
             if track.get('title'):
-                # Track title may be distinct from underling recording
+                # Track title may be distinct from underlying recording
                 # title.
                 ti.title = track['title']
             ti.disctitle = disctitle
@@ -173,27 +214,36 @@ def album_info(release):
         mediums=len(release['medium-list']),
         artist_sort=artist_sort_name,
         artist_credit=artist_credit_name,
+        data_source='MusicBrainz',
     )
     info.va = info.artist_id == VARIOUS_ARTISTS_ID
     info.asin = release.get('asin')
     info.releasegroup_id = release['release-group']['id']
-    info.albumdisambig = release['release-group'].get('disambiguation')
     info.country = release.get('country')
     info.albumstatus = release.get('status')
 
+    # Build up the disambiguation string from the release group and release.
+    disambig = []
+    if release['release-group'].get('disambiguation'):
+        disambig.append(release['release-group'].get('disambiguation'))
+    if release.get('disambiguation'):
+        disambig.append(release.get('disambiguation'))
+    info.albumdisambig = u', '.join(disambig)
+
     # Release type not always populated.
     if 'type' in release['release-group']:
         reltype = release['release-group']['type']
         if reltype:
             info.albumtype = reltype.lower()
 
-    # Release date.
-    if 'first-release-date' in release['release-group']:
-        # Try earliest release date for the entire group first.
-        _set_date_str(info, release['release-group']['first-release-date'])
-    elif 'date' in release:
-        # Fall back to release-specific date.
-        _set_date_str(info, release['date'])
+    # Release dates.
+    release_date = release.get('date')
+    release_group_date = release['release-group'].get('first-release-date')
+    if not release_date:
+        # Fall back if release-specific date is not available.
+        release_date = release_group_date
+    _set_date_str(info, release_date, False)
+    _set_date_str(info, release_group_date, True)
 
     # Label name.
     if release.get('label-info-list'):
@@ -241,7 +291,7 @@ def match_album(artist, album, tracks=None, limit=SEARCH_LIMIT):
         return
 
     try:
-        res = _mb_release_search(limit=limit, **criteria)
+        res = musicbrainzngs.search_releases(limit=limit, **criteria)
     except musicbrainzngs.MusicBrainzError as exc:
         raise MusicBrainzAPIError(exc, 'release search', criteria,
                                   traceback.format_exc())
@@ -265,20 +315,34 @@ def match_track(artist, title, limit=SEARCH_LIMIT):
         return
 
     try:
-        res = _mb_recording_search(limit=limit, **criteria)
+        res = musicbrainzngs.search_recordings(limit=limit, **criteria)
     except musicbrainzngs.MusicBrainzError as exc:
         raise MusicBrainzAPIError(exc, 'recording search', criteria,
                                   traceback.format_exc())
     for recording in res['recording-list']:
         yield track_info(recording)
 
+def _parse_id(s):
+    """Search for a MusicBrainz ID in the given string and return it. If
+    no ID can be found, return None.
+    """
+    # Find the first thing that looks like a UUID/MBID.
+    match = re.search('[a-f0-9]{8}(-[a-f0-9]{4}){3}-[a-f0-9]{12}', s)
+    if match:
+        return match.group()
+
 def album_for_id(albumid):
     """Fetches an album by its MusicBrainz ID and returns an AlbumInfo
     object or None if the album is not found. May raise a
     MusicBrainzAPIError.
     """
+    albumid = _parse_id(albumid)
+    if not albumid:
+        log.error('Invalid MBID.')
+        return
     try:
-        res = musicbrainzngs.get_release_by_id(albumid, RELEASE_INCLUDES)
+        res = musicbrainzngs.get_release_by_id(albumid,
+                                               RELEASE_INCLUDES)
     except musicbrainzngs.ResponseError:
         log.debug('Album ID match failed.')
         return None
@@ -291,6 +355,10 @@ def track_for_id(trackid):
     """Fetches a track by its MusicBrainz ID. Returns a TrackInfo object
     or None if no track is found. May raise a MusicBrainzAPIError.
     """
+    trackid = _parse_id(trackid)
+    if not trackid:
+        log.error('Invalid MBID.')
+        return
     try:
         res = musicbrainzngs.get_recording_by_id(trackid, TRACK_INCLUDES)
     except musicbrainzngs.ResponseError:
