@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2010, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -29,12 +29,9 @@ import beets
 from beets.plugins import BeetsPlugin
 import beets.ui
 from beets import vfs
+from beets import config
 from beets.util import bluelet
 
-
-DEFAULT_PORT = 6600
-DEFAULT_HOST = ''
-DEFAULT_PASSWORD = ''
 PROTOCOL_VERSION = '0.13.0'
 BUFSIZE = 1024
 
@@ -154,8 +151,7 @@ class BaseServer(object):
     This is a generic superclass and doesn't support many commands.
     """
 
-    def __init__(self, host=DEFAULT_HOST, port=DEFAULT_PORT,
-                 password=DEFAULT_PASSWORD):
+    def __init__(self, host, port, password):
         """Create a new server bound to address `host` and listening
         on port `port`. If `password` is given, it is required to do
         anything significant on the server.
@@ -727,7 +723,7 @@ class Server(BaseServer):
     to store its library.
     """
 
-    def __init__(self, library, host='', port=DEFAULT_PORT, password=''):
+    def __init__(self, library, host, port, password):
         try:
             from beetsplug.bpd import gstplayer
         except ImportError as e:
@@ -923,12 +919,12 @@ class Server(BaseServer):
             item = self.playlist[self.current_index]
 
             yield u'bitrate: ' + unicode(item.bitrate/1000)
-            #fixme: missing 'audio'
+            # Missing 'audio'.
 
             (pos, total) = self.player.time()
             yield u'time: ' + unicode(pos) + u':' + unicode(total)
 
-        #fixme: also missing 'updating_db'
+        # Also missing 'updating_db'.
 
 
     def cmd_stats(self, conn):
@@ -945,7 +941,7 @@ class Server(BaseServer):
                u'albums: ' + unicode(albums),
                u'songs: ' + unicode(songs),
                u'uptime: ' + unicode(int(time.time() - self.startup_time)),
-               u'playtime: ' + u'0', #fixme
+               u'playtime: ' + u'0', # Missing.
                u'db_playtime: ' + unicode(int(totaltime)),
                u'db_update: ' + unicode(int(self.updated_time)),
               )
@@ -954,17 +950,19 @@ class Server(BaseServer):
     # Searching.
 
     tagtype_map = {
-        u'Artist':       u'artist',
-        u'Album':        u'album',
-        u'Title':        u'title',
-        u'Track':        u'track',
+        u'Artist':          u'artist',
+        u'Album':           u'album',
+        u'Title':           u'title',
+        u'Track':           u'track',
+        u'AlbumArtist':     u'albumartist',
+        u'AlbumArtistSort': u'albumartist_sort',
         # Name?
-        u'Genre':        u'genre',
-        u'Date':         u'year',
-        u'Composer':     u'composer',
+        u'Genre':           u'genre',
+        u'Date':            u'year',
+        u'Composer':        u'composer',
         # Performer?
-        u'Disc':         u'disc',
-        u'filename':     u'path', # Suspect.
+        u'Disc':            u'disc',
+        u'filename':        u'path', # Suspect.
     }
 
     def cmd_tagtypes(self, conn):
@@ -1125,6 +1123,14 @@ class BPDPlugin(BeetsPlugin):
     """Provides the "beet bpd" command for running a music player
     server.
     """
+    def __init__(self):
+        super(BPDPlugin, self).__init__()
+        self.config.add({
+            'host': u'',
+            'port': 6600,
+            'password': u'',
+        })
+
     def start_bpd(self, lib, host, port, password, debug):
         """Starts a BPD server."""
         if debug:
@@ -1144,15 +1150,12 @@ class BPDPlugin(BeetsPlugin):
         cmd.parser.add_option('-d', '--debug', action='store_true',
             help='dump all MPD traffic to stdout')
 
-        def func(lib, config, opts, args):
-            host = args.pop(0) if args else \
-                beets.ui.config_val(config, 'bpd', 'host', DEFAULT_HOST)
-            port = args.pop(0) if args else \
-                beets.ui.config_val(config, 'bpd', 'port', str(DEFAULT_PORT))
+        def func(lib, opts, args):
+            host = args.pop(0) if args else self.config['host'].get(unicode)
+            port = args.pop(0) if args else self.config['port'].get(int)
             if args:
                 raise beets.ui.UserError('too many arguments')
-            password = beets.ui.config_val(config, 'bpd', 'password',
-                                           DEFAULT_PASSWORD)
+            password = self.config['password'].get(unicode)
             debug = opts.debug or False
             self.start_bpd(lib, host, int(port), password, debug)
 
