@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2012, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,16 +20,17 @@ from beets.plugins import BeetsPlugin
 from beets import mediafile
 from beets import ui
 from beets.ui import decargs
-from beets.util import syspath, normpath
+from beets.util import syspath, normpath, displayable_path
 from beets.util.artresizer import ArtResizer
+from beets import config
 
 log = logging.getLogger('beets')
 
-def _embed(path, items):
+def _embed(path, items, maxwidth=0):
     """Embed an image file, located at `path`, into each item.
     """
-    if options['maxwidth']:
-        path = ArtResizer.shared.resize(options['maxwidth'], syspath(path))
+    if maxwidth:
+        path = ArtResizer.shared.resize(maxwidth, syspath(path))
 
     data = open(syspath(path), 'rb').read()
     kindstr = imghdr.what(None, data)
@@ -45,39 +46,39 @@ def _embed(path, items):
             f = mediafile.MediaFile(syspath(item.path))
         except mediafile.UnreadableFileError as exc:
             log.warn('Could not embed art in {0}: {1}'.format(
-                repr(item.path), exc
+                displayable_path(item.path), exc
             ))
             continue
         f.art = data
         f.save()
 
-options = {
-    'autoembed': True,
-    'maxwidth': 0,
-}
 class EmbedCoverArtPlugin(BeetsPlugin):
     """Allows albumart to be embedded into the actual files.
     """
-    def configure(self, config):
-        options['autoembed'] = \
-            ui.config_val(config, 'embedart', 'autoembed', True, bool)
-        options['maxwidth'] = \
-            int(ui.config_val(config, 'embedart', 'maxwidth', '0'))
-
-        if options['maxwidth'] and not ArtResizer.shared.local:
-            options['maxwidth'] = 0
-            log.error("embedart: ImageMagick or PIL not found; "
-                      "'maxwidth' option ignored")
+    def __init__(self):
+        super(EmbedCoverArtPlugin, self).__init__()
+        self.config.add({
+            'maxwidth': 0,
+            'auto': True,
+        })
+        if self.config['maxwidth'].get(int) and \
+                not ArtResizer.shared.local:
+            self.config['maxwidth'] = 0
+            log.warn("embedart: ImageMagick or PIL not found; "
+                     "'maxwidth' option ignored")
 
     def commands(self):
         # Embed command.
         embed_cmd = ui.Subcommand('embedart',
-                                  help='embed an image file into file metadata')
-        def embed_func(lib, config, opts, args):
-            if not args:
-                raise ui.UserError('specify an image file')
-            imagepath = normpath(args.pop(0))
-            embed(lib, imagepath, decargs(args))
+            help='embed image files into file metadata')
+        embed_cmd.parser.add_option('-f', '--file', metavar='PATH',
+            help='the image file to embed')
+        def embed_func(lib, opts, args):
+            if opts.file:
+                imagepath = normpath(opts.file)
+                embed(lib, imagepath, decargs(args))
+            else:
+                embed_current(lib, decargs(args))
         embed_cmd.func = embed_func
 
         # Extract command.
@@ -85,7 +86,7 @@ class EmbedCoverArtPlugin(BeetsPlugin):
                                     help='extract an image from file metadata')
         extract_cmd.parser.add_option('-o', dest='outpath',
                                       help='image output file')
-        def extract_func(lib, config, opts, args):
+        def extract_func(lib, opts, args):
             outpath = normpath(opts.outpath or 'cover')
             extract(lib, outpath, decargs(args))
         extract_cmd.func = extract_func
@@ -93,13 +94,13 @@ class EmbedCoverArtPlugin(BeetsPlugin):
         # Clear command.
         clear_cmd = ui.Subcommand('clearart',
                                   help='remove images from file metadata')
-        def clear_func(lib, config, opts, args):
+        def clear_func(lib, opts, args):
             clear(lib, decargs(args))
         clear_cmd.func = clear_func
 
         return [embed_cmd, extract_cmd, clear_cmd]
 
-# "embedart" command.
+# "embedart" command with --file argument.
 def embed(lib, imagepath, query):
     albums = lib.albums(query)
     for i_album in albums:
@@ -111,7 +112,22 @@ def embed(lib, imagepath, query):
 
     log.info('Embedding album art into %s - %s.' % \
              (album.albumartist, album.album))
-    _embed(imagepath, album.items())
+    _embed(imagepath, album.items(),
+           config['embedart']['maxwidth'].get(int))
+
+# "embedart" command without explicit file.
+def embed_current(lib, query):
+    albums = lib.albums(query)
+    for album in albums:
+        if not album.artpath:
+            log.info(u'No album art present: {0} - {1}'.
+                     format(album.albumartist, album.album))
+            continue
+
+        log.info(u'Embedding album art into {0} - {1}'.
+                 format(album.albumartist, album.album))
+        _embed(album.artpath, album.items(),
+               config['embedart']['maxwidth'].get(int))
 
 # "extractart" command.
 def extract(lib, outpath, query):
@@ -124,7 +140,14 @@ def extract(lib, outpath, query):
         return
 
     # Extract the art.
-    mf = mediafile.MediaFile(syspath(item.path))
+    try:
+        mf = mediafile.MediaFile(syspath(item.path))
+    except mediafile.UnreadableFileError as exc:
+        log.error(u'Could not extract art from {0}: {1}'.format(
+            displayable_path(item.path), exc
+        ))
+        return
+
     art = mf.art
     if not art:
         log.error('No album art present in %s - %s.' %
@@ -149,12 +172,19 @@ def clear(lib, query):
     log.info('Clearing album art from items:')
     for item in lib.items(query):
         log.info(u'%s - %s' % (item.artist, item.title))
-        mf = mediafile.MediaFile(syspath(item.path))
+        try:
+            mf = mediafile.MediaFile(syspath(item.path))
+        except mediafile.UnreadableFileError as exc:
+            log.error(u'Could not clear art from {0}: {1}'.format(
+                displayable_path(item.path), exc
+            ))
+            continue
         mf.art = None
         mf.save()
 
 # Automatically embed art into imported albums.
 @EmbedCoverArtPlugin.listen('album_imported')
-def album_imported(lib, album, config):
-    if album.artpath and options['autoembed']:
-        _embed(album.artpath, album.items())
+def album_imported(lib, album):
+    if album.artpath and config['embedart']['auto']:
+        _embed(album.artpath, album.items(),
+               config['embedart']['maxwidth'].get(int))
