@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2012, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -58,7 +58,7 @@ def info(paths):
 class InfoPlugin(BeetsPlugin):
     def commands(self):
         cmd = ui.Subcommand('info', help='show file metadata')
-        def func(lib, config, opts, args):
+        def func(lib, opts, args):
             if not args:
                 raise ui.UserError('no file specified')
             info(args)
