@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2012, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,10 +20,10 @@ import logging
 from beets.plugins import BeetsPlugin
 from beets import ui
 from beets import util
+from beets import config
 
 log = logging.getLogger('beets')
 
-AUTOSCRUB_KEY = 'autoscrub'
 _MUTAGEN_FORMATS = {
     'asf': 'ASF',
     'apev2': 'APEv2File',
@@ -42,17 +42,16 @@ _MUTAGEN_FORMATS = {
 
 scrubbing = False
 
-options = {
-    AUTOSCRUB_KEY: True,
-}
 class ScrubPlugin(BeetsPlugin):
     """Removes extraneous metadata from files' tags."""
-    def configure(self, config):
-        options[AUTOSCRUB_KEY] = \
-            ui.config_val(config, 'scrub', AUTOSCRUB_KEY, True, bool)
+    def __init__(self):
+        super(ScrubPlugin, self).__init__()
+        self.config.add({
+            'auto': True,
+        })
 
     def commands(self):
-        def scrub_func(lib, config, opts, args):
+        def scrub_func(lib, opts, args):
             # This is a little bit hacky, but we set a global flag to
             # avoid autoscrubbing when we're also explicitly scrubbing.
             global scrubbing
@@ -101,12 +100,24 @@ def _scrub(path):
             continue
 
         # Remove the tag for this type.
-        f.delete()
+        try:
+            f.delete()
+        except NotImplementedError:
+            # Some Mutagen metadata subclasses (namely, ASFTag) do not
+            # support .delete(), presumably because it is impossible to
+            # remove them. In this case, we just remove all the tags.
+            for tag in f.keys():
+                del f[tag]
+        except IOError as exc:
+            log.error(u'could not scrub {0}: {1}'.format(
+                util.displayable_path(path),
+                exc,
+            ))
         f.save()
 
 # Automatically embed art into imported albums.
 @ScrubPlugin.listen('write')
 def write_item(item):
-    if not scrubbing and options[AUTOSCRUB_KEY]:
+    if not scrubbing and config['scrub']['auto']:
         log.debug(u'auto-scrubbing %s' % util.displayable_path(item.path))
         _scrub(item.path)
