@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2011, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -23,11 +23,13 @@ import _common
 from _common import unittest
 from beets import autotag
 from beets.autotag import match
+from beets.autotag.hooks import Distance, string_dist
 from beets.library import Item
 from beets.util import plurality
 from beets.autotag import AlbumInfo, TrackInfo
+from beets import config
 
-class PluralityTest(unittest.TestCase):
+class PluralityTest(_common.TestCase):
     def test_plurality_consensus(self):
         objs = [1, 1, 1, 1]
         obj, freq = plurality(objs)
@@ -54,19 +56,19 @@ class PluralityTest(unittest.TestCase):
         items = [Item({'artist': 'The Beetles', 'album': 'The White Album'}),
                  Item({'artist': 'The Beatles', 'album': 'The White Album'}),
                  Item({'artist': 'The Beatles', 'album': 'Teh White Album'})]
-        l_artist, l_album, artist_consensus = match.current_metadata(items)
-        self.assertEqual(l_artist, 'The Beatles')
-        self.assertEqual(l_album, 'The White Album')
-        self.assertFalse(artist_consensus)
+        likelies, consensus = match.current_metadata(items)
+        self.assertEqual(likelies['artist'], 'The Beatles')
+        self.assertEqual(likelies['album'], 'The White Album')
+        self.assertFalse(consensus['artist'])
 
     def test_current_metadata_artist_consensus(self):
         items = [Item({'artist': 'The Beatles', 'album': 'The White Album'}),
                  Item({'artist': 'The Beatles', 'album': 'The White Album'}),
                  Item({'artist': 'The Beatles', 'album': 'Teh White Album'})]
-        l_artist, l_album, artist_consensus = match.current_metadata(items)
-        self.assertEqual(l_artist, 'The Beatles')
-        self.assertEqual(l_album, 'The White Album')
-        self.assertTrue(artist_consensus)
+        likelies, consensus = match.current_metadata(items)
+        self.assertEqual(likelies['artist'], 'The Beatles')
+        self.assertEqual(likelies['album'], 'The White Album')
+        self.assertTrue(consensus['artist'])
 
     def test_albumartist_consensus(self):
         items = [Item({'artist': 'tartist1', 'album': 'album',
@@ -75,9 +77,19 @@ class PluralityTest(unittest.TestCase):
                        'albumartist': 'aartist'}),
                  Item({'artist': 'tartist3', 'album': 'album',
                        'albumartist': 'aartist'})]
-        l_artist, l_album, artist_consensus = match.current_metadata(items)
-        self.assertEqual(l_artist, 'aartist')
-        self.assertFalse(artist_consensus)
+        likelies, consensus = match.current_metadata(items)
+        self.assertEqual(likelies['artist'], 'aartist')
+        self.assertFalse(consensus['artist'])
+
+    def test_current_metadata_likelies(self):
+        fields = ['artist', 'album', 'albumartist', 'year', 'disctotal',
+                  'mb_albumid', 'label', 'catalognum', 'country', 'media',
+                  'albumdisambig']
+        items = [Item(dict((f, '%s_%s' % (f, i or 1)) for f in fields))
+                 for i in range(5)]
+        likelies, _ = match.current_metadata(items)
+        for f in fields:
+            self.assertEqual(likelies[f], '%s_1' % f)
 
 def _make_item(title, track, artist=u'some artist'):
     return Item({
@@ -94,7 +106,163 @@ def _make_trackinfo():
         TrackInfo(u'three', None, u'some artist', length=1, index=3),
     ]
 
-class TrackDistanceTest(unittest.TestCase):
+class DistanceTest(_common.TestCase):
+    def test_add(self):
+        dist = Distance()
+        dist.add('add', 1.0)
+        self.assertEqual(dist._penalties, {'add': [1.0]})
+
+    def test_add_equality(self):
+        dist = Distance()
+        dist.add_equality('equality', 'ghi', ['abc', 'def', 'ghi'])
+        self.assertEqual(dist._penalties['equality'], [0.0])
+
+        dist.add_equality('equality', 'xyz', ['abc', 'def', 'ghi'])
+        self.assertEqual(dist._penalties['equality'], [0.0, 1.0])
+
+        dist.add_equality('equality', 'abc', re.compile(r'ABC', re.I))
+        self.assertEqual(dist._penalties['equality'], [0.0, 1.0, 0.0])
+
+    def test_add_expr(self):
+        dist = Distance()
+        dist.add_expr('expr', True)
+        self.assertEqual(dist._penalties['expr'], [1.0])
+
+        dist.add_expr('expr', False)
+        self.assertEqual(dist._penalties['expr'], [1.0, 0.0])
+
+    def test_add_number(self):
+        dist = Distance()
+        # Add a full penalty for each number of difference between two numbers.
+
+        dist.add_number('number', 1, 1)
+        self.assertEqual(dist._penalties['number'], [0.0])
+
+        dist.add_number('number', 1, 2)
+        self.assertEqual(dist._penalties['number'], [0.0, 1.0])
+
+        dist.add_number('number', 2, 1)
+        self.assertEqual(dist._penalties['number'], [0.0, 1.0, 1.0])
+
+        dist.add_number('number', -1, 2)
+        self.assertEqual(dist._penalties['number'], [0.0, 1.0, 1.0, 1.0,
+                                                          1.0, 1.0])
+
+    def test_add_priority(self):
+        dist = Distance()
+        dist.add_priority('priority', 'abc', 'abc')
+        self.assertEqual(dist._penalties['priority'], [0.0])
+
+        dist.add_priority('priority', 'def', ['abc', 'def'])
+        self.assertEqual(dist._penalties['priority'], [0.0, 0.5])
+
+        dist.add_priority('priority', 'gh', ['ab', 'cd', 'ef',
+                                                  re.compile('GH', re.I)])
+        self.assertEqual(dist._penalties['priority'], [0.0, 0.5, 0.75])
+
+        dist.add_priority('priority', 'xyz', ['abc', 'def'])
+        self.assertEqual(dist._penalties['priority'], [0.0, 0.5, 0.75,
+                                                            1.0])
+
+    def test_add_ratio(self):
+        dist = Distance()
+        dist.add_ratio('ratio', 25, 100)
+        self.assertEqual(dist._penalties['ratio'], [0.25])
+
+        dist.add_ratio('ratio', 10, 5)
+        self.assertEqual(dist._penalties['ratio'], [0.25, 1.0])
+
+        dist.add_ratio('ratio', -5, 5)
+        self.assertEqual(dist._penalties['ratio'], [0.25, 1.0, 0.0])
+
+        dist.add_ratio('ratio', 5, 0)
+        self.assertEqual(dist._penalties['ratio'], [0.25, 1.0, 0.0, 0.0])
+
+    def test_add_string(self):
+        dist = Distance()
+        sdist = string_dist(u'abc', u'bcd')
+        dist.add_string('string', u'abc', u'bcd')
+        self.assertEqual(dist._penalties['string'], [sdist])
+
+    def test_distance(self):
+        config['match']['distance_weights']['album'] = 2.0
+        config['match']['distance_weights']['medium'] = 1.0
+        dist = Distance()
+        dist.add('album', 0.5)
+        dist.add('media', 0.25)
+        dist.add('media', 0.75)
+        self.assertEqual(dist.distance, 0.5)
+
+        # __getitem__()
+        self.assertEqual(dist['album'], 0.25)
+        self.assertEqual(dist['media'], 0.25)
+
+    def test_max_distance(self):
+        config['match']['distance_weights']['album'] = 3.0
+        config['match']['distance_weights']['medium'] = 1.0
+        dist = Distance()
+        dist.add('album', 0.5)
+        dist.add('medium', 0.0)
+        dist.add('medium', 0.0)
+        self.assertEqual(dist.max_distance, 5.0)
+
+    def test_operators(self):
+        config['match']['distance_weights']['source'] = 1.0
+        config['match']['distance_weights']['album'] = 2.0
+        config['match']['distance_weights']['medium'] = 1.0
+        dist = Distance()
+        dist.add('source', 0.0)
+        dist.add('album', 0.5)
+        dist.add('medium', 0.25)
+        dist.add('medium', 0.75)
+        self.assertEqual(len(dist), 2)
+        self.assertEqual(list(dist), [('album', 0.2), ('medium', 0.2)])
+        self.assertTrue(dist == 0.4)
+        self.assertTrue(dist < 1.0)
+        self.assertTrue(dist > 0.0)
+        self.assertEqual(dist - 0.4, 0.0)
+        self.assertEqual(0.4 - dist, 0.0)
+        self.assertEqual(float(dist), 0.4)
+
+    def test_raw_distance(self):
+        config['match']['distance_weights']['album'] = 3.0
+        config['match']['distance_weights']['medium'] = 1.0
+        dist = Distance()
+        dist.add('album', 0.5)
+        dist.add('medium', 0.25)
+        dist.add('medium', 0.5)
+        self.assertEqual(dist.raw_distance, 2.25)
+
+    def test_items(self):
+        config['match']['distance_weights']['album'] = 4.0
+        config['match']['distance_weights']['medium'] = 2.0
+        dist = Distance()
+        dist.add('album', 0.1875)
+        dist.add('medium', 0.75)
+        self.assertEqual(dist.items(), [('medium', 0.25), ('album', 0.125)])
+
+        # Sort by key if distance is equal.
+        dist = Distance()
+        dist.add('album', 0.375)
+        dist.add('medium', 0.75)
+        self.assertEqual(dist.items(), [('album', 0.25), ('medium', 0.25)])
+
+    def test_update(self):
+        dist1 = Distance()
+        dist1.add('album', 0.5)
+        dist1.add('media', 1.0)
+
+        dist2 = Distance()
+        dist2.add('album', 0.75)
+        dist2.add('album', 0.25)
+        dist2.add('media', 0.05)
+
+        dist1.update(dist2)
+
+        self.assertEqual(dist1._penalties, {'album': [0.5, 0.75, 0.25],
+                                             'media': [1.0, 0.05]})
+
+class TrackDistanceTest(_common.TestCase):
     def test_identical_tracks(self):
         item = _make_item(u'one', 1)
         info = _make_trackinfo()[0]
@@ -121,7 +289,7 @@ class TrackDistanceTest(unittest.TestCase):
         dist = match.track_distance(item, info, incl_artist=True)
         self.assertEqual(dist, 0.0)
 
-class AlbumDistanceTest(unittest.TestCase):
+class AlbumDistanceTest(_common.TestCase):
     def _mapping(self, items, info):
         out = {}
         for i, t in zip(items, info.tracks):
@@ -274,10 +442,12 @@ class AlbumDistanceTest(unittest.TestCase):
 
 def _mkmp3(path):
     shutil.copyfile(os.path.join(_common.RSRC, 'min.mp3'), path)
-class AlbumsInDirTest(unittest.TestCase):
+class AlbumsInDirTest(_common.TestCase):
     def setUp(self):
+        super(AlbumsInDirTest, self).setUp()
+
         # create a directory structure for testing
-        self.base = os.path.abspath(os.path.join(_common.RSRC, 'tempdir'))
+        self.base = os.path.abspath(os.path.join(self.temp_dir, 'tempdir'))
         os.mkdir(self.base)
 
         os.mkdir(os.path.join(self.base, 'album1'))
@@ -291,8 +461,6 @@ class AlbumsInDirTest(unittest.TestCase):
         _mkmp3(os.path.join(self.base, 'album2', 'album2song.mp3'))
         _mkmp3(os.path.join(self.base, 'more', 'album3', 'album3song.mp3'))
         _mkmp3(os.path.join(self.base, 'more', 'album4', 'album4song.mp3'))
-    def tearDown(self):
-        shutil.rmtree(self.base)
 
     def test_finds_all_albums(self):
         albums = list(autotag.albums_in_dir(self.base))
@@ -315,25 +483,42 @@ class AlbumsInDirTest(unittest.TestCase):
             else:
                 self.assertEqual(len(album), 1)
 
-class MultiDiscAlbumsInDirTest(unittest.TestCase):
+class MultiDiscAlbumsInDirTest(_common.TestCase):
     def setUp(self):
-        self.base = os.path.abspath(os.path.join(_common.RSRC, 'tempdir'))
+        super(MultiDiscAlbumsInDirTest, self).setUp()
+
+        self.base = os.path.abspath(os.path.join(self.temp_dir, 'tempdir'))
         os.mkdir(self.base)
 
         self.dirs = [
-            os.path.join(self.base, 'album1'),
-            os.path.join(self.base, 'album1', 'disc 1'),
-            os.path.join(self.base, 'album1', 'disc 2'),
-            os.path.join(self.base, 'dir2'),
-            os.path.join(self.base, 'dir2', 'disc 1'),
-            os.path.join(self.base, 'dir2', 'something'),
+            # Nested album, multiple subdirs.
+            # Also, false positive marker in root dir, and subtitle for disc 3.
+            os.path.join(self.base, 'ABCD1234'),
+            os.path.join(self.base, 'ABCD1234', 'cd 1'),
+            os.path.join(self.base, 'ABCD1234', 'cd 3 - bonus'),
+
+            # Nested album, single subdir.
+            # Also, punctuation between marker and disc number.
+            os.path.join(self.base, 'album'),
+            os.path.join(self.base, 'album', 'cd _ 1'),
+
+            # Flattened album, case typo.
+            # Also, false positive marker in parent dir.
+            os.path.join(self.base, 'artist [CD5]'),
+            os.path.join(self.base, 'artist [CD5]', 'CAT disc 1'),
+            os.path.join(self.base, 'artist [CD5]', 'CAt disc 2'),
+
+            # Single disc album, sorted between CAT discs.
+            os.path.join(self.base, 'artist [CD5]', 'CATS'),
         ]
         self.files = [
-            os.path.join(self.base, 'album1', 'disc 1', 'song1.mp3'),
-            os.path.join(self.base, 'album1', 'disc 2', 'song2.mp3'),
-            os.path.join(self.base, 'album1', 'disc 2', 'song3.mp3'),
-            os.path.join(self.base, 'dir2', 'disc 1', 'song4.mp3'),
-            os.path.join(self.base, 'dir2', 'something', 'song5.mp3'),
+            os.path.join(self.base, 'ABCD1234', 'cd 1', 'song1.mp3'),
+            os.path.join(self.base, 'ABCD1234', 'cd 3 - bonus', 'song2.mp3'),
+            os.path.join(self.base, 'ABCD1234', 'cd 3 - bonus', 'song3.mp3'),
+            os.path.join(self.base, 'album', 'cd _ 1', 'song4.mp3'),
+            os.path.join(self.base, 'artist [CD5]', 'CAT disc 1', 'song5.mp3'),
+            os.path.join(self.base, 'artist [CD5]', 'CAt disc 2', 'song6.mp3'),
+            os.path.join(self.base, 'artist [CD5]', 'CATS', 'song7.mp3'),
         ]
 
         for path in self.dirs:
@@ -341,28 +526,35 @@ class MultiDiscAlbumsInDirTest(unittest.TestCase):
         for path in self.files:
             _mkmp3(path)
 
-    def tearDown(self):
-        shutil.rmtree(self.base)
-
-    def test_coalesce_multi_disc_album(self):
+    def test_coalesce_nested_album_multiple_subdirs(self):
         albums = list(autotag.albums_in_dir(self.base))
-        self.assertEquals(len(albums), 3)
+        self.assertEquals(len(albums), 4)
         root, items = albums[0]
-        self.assertEquals(root, os.path.join(self.base, 'album1'))
+        self.assertEquals(root, self.dirs[0:3])
         self.assertEquals(len(items), 3)
 
-    def test_separate_red_herring(self):
+    def test_coalesce_nested_album_single_subdir(self):
         albums = list(autotag.albums_in_dir(self.base))
         root, items = albums[1]
-        self.assertEquals(root, os.path.join(self.base, 'dir2', 'disc 1'))
+        self.assertEquals(root, self.dirs[3:5])
+        self.assertEquals(len(items), 1)
+
+    def test_coalesce_flattened_album_case_typo(self):
+        albums = list(autotag.albums_in_dir(self.base))
         root, items = albums[2]
-        self.assertEquals(root, os.path.join(self.base, 'dir2', 'something'))
+        self.assertEquals(root, self.dirs[6:8])
+        self.assertEquals(len(items), 2)
+
+    def test_single_disc_album(self):
+        albums = list(autotag.albums_in_dir(self.base))
+        root, items = albums[3]
+        self.assertEquals(root, self.dirs[8:])
+        self.assertEquals(len(items), 1)
 
     def test_do_not_yield_empty_album(self):
         # Remove all the MP3s.
         for path in self.files:
             os.remove(path)
-
         albums = list(autotag.albums_in_dir(self.base))
         self.assertEquals(len(albums), 0)
 
@@ -384,8 +576,8 @@ class AssignmentTest(unittest.TestCase):
         trackinfo.append(TrackInfo(u'three', None))
         mapping, extra_items, extra_tracks = \
             match.assign_items(items, trackinfo)
-        self.assertEqual(extra_items, set())
-        self.assertEqual(extra_tracks, set())
+        self.assertEqual(extra_items, [])
+        self.assertEqual(extra_tracks, [])
         self.assertEqual(mapping, {
             items[0]: trackinfo[0],
             items[1]: trackinfo[2],
@@ -403,8 +595,8 @@ class AssignmentTest(unittest.TestCase):
         trackinfo.append(TrackInfo(u'three', None))
         mapping, extra_items, extra_tracks = \
             match.assign_items(items, trackinfo)
-        self.assertEqual(extra_items, set())
-        self.assertEqual(extra_tracks, set())
+        self.assertEqual(extra_items, [])
+        self.assertEqual(extra_tracks, [])
         self.assertEqual(mapping, {
             items[0]: trackinfo[0],
             items[1]: trackinfo[2],
@@ -421,8 +613,8 @@ class AssignmentTest(unittest.TestCase):
         trackinfo.append(TrackInfo(u'three', None))
         mapping, extra_items, extra_tracks = \
             match.assign_items(items, trackinfo)
-        self.assertEqual(extra_items, set())
-        self.assertEqual(extra_tracks, set([trackinfo[1]]))
+        self.assertEqual(extra_items, [])
+        self.assertEqual(extra_tracks, [trackinfo[1]])
         self.assertEqual(mapping, {
             items[0]: trackinfo[0],
             items[1]: trackinfo[2],
@@ -438,8 +630,8 @@ class AssignmentTest(unittest.TestCase):
         trackinfo.append(TrackInfo(u'three', None))
         mapping, extra_items, extra_tracks = \
             match.assign_items(items, trackinfo)
-        self.assertEqual(extra_items, set([items[1]]))
-        self.assertEqual(extra_tracks, set())
+        self.assertEqual(extra_items, [items[1]])
+        self.assertEqual(extra_tracks, [])
         self.assertEqual(mapping, {
             items[0]: trackinfo[0],
             items[2]: trackinfo[1],
@@ -488,8 +680,8 @@ class AssignmentTest(unittest.TestCase):
 
         mapping, extra_items, extra_tracks = \
             match.assign_items(items, trackinfo)
-        self.assertEqual(extra_items, set())
-        self.assertEqual(extra_tracks, set())
+        self.assertEqual(extra_items, [])
+        self.assertEqual(extra_tracks, [])
         for item, info in mapping.iteritems():
             self.assertEqual(items.index(item), trackinfo.index(info))
 
@@ -499,10 +691,13 @@ class ApplyTestUtil(object):
         mapping = {}
         for i, t in zip(self.items, info.tracks):
             mapping[i] = t
-        autotag.apply_metadata(info, mapping, per_disc_numbering)
+        config['per_disc_numbering'] = per_disc_numbering
+        autotag.apply_metadata(info, mapping)
 
-class ApplyTest(unittest.TestCase, ApplyTestUtil):
+class ApplyTest(_common.TestCase, ApplyTestUtil):
     def setUp(self):
+        super(ApplyTest, self).setUp()
+
         self.items = []
         self.items.append(Item({}))
         self.items.append(Item({}))
@@ -613,8 +808,10 @@ class ApplyTest(unittest.TestCase, ApplyTestUtil):
         self.assertEqual(self.items[1].albumartist_sort, 'albumArtistSort')
         self.assertEqual(self.items[1].artist_sort, 'albumArtistSort')
 
-class ApplyCompilationTest(unittest.TestCase, ApplyTestUtil):
+class ApplyCompilationTest(_common.TestCase, ApplyTestUtil):
     def setUp(self):
+        super(ApplyCompilationTest, self).setUp()
+
         self.items = []
         self.items.append(Item({}))
         self.items.append(Item({}))
@@ -675,77 +872,77 @@ class ApplyCompilationTest(unittest.TestCase, ApplyTestUtil):
 
 class StringDistanceTest(unittest.TestCase):
     def test_equal_strings(self):
-        dist = match.string_dist(u'Some String', u'Some String')
+        dist = string_dist(u'Some String', u'Some String')
         self.assertEqual(dist, 0.0)
 
     def test_different_strings(self):
-        dist = match.string_dist(u'Some String', u'Totally Different')
+        dist = string_dist(u'Some String', u'Totally Different')
         self.assertNotEqual(dist, 0.0)
 
     def test_punctuation_ignored(self):
-        dist = match.string_dist(u'Some String', u'Some.String!')
+        dist = string_dist(u'Some String', u'Some.String!')
         self.assertEqual(dist, 0.0)
 
     def test_case_ignored(self):
-        dist = match.string_dist(u'Some String', u'sOME sTring')
+        dist = string_dist(u'Some String', u'sOME sTring')
         self.assertEqual(dist, 0.0)
 
     def test_leading_the_has_lower_weight(self):
-        dist1 = match.string_dist(u'XXX Band Name', u'Band Name')
-        dist2 = match.string_dist(u'The Band Name', u'Band Name')
+        dist1 = string_dist(u'XXX Band Name', u'Band Name')
+        dist2 = string_dist(u'The Band Name', u'Band Name')
         self.assert_(dist2 < dist1)
 
     def test_parens_have_lower_weight(self):
-        dist1 = match.string_dist(u'One .Two.', u'One')
-        dist2 = match.string_dist(u'One (Two)', u'One')
+        dist1 = string_dist(u'One .Two.', u'One')
+        dist2 = string_dist(u'One (Two)', u'One')
         self.assert_(dist2 < dist1)
 
     def test_brackets_have_lower_weight(self):
-        dist1 = match.string_dist(u'One .Two.', u'One')
-        dist2 = match.string_dist(u'One [Two]', u'One')
+        dist1 = string_dist(u'One .Two.', u'One')
+        dist2 = string_dist(u'One [Two]', u'One')
         self.assert_(dist2 < dist1)
 
     def test_ep_label_has_zero_weight(self):
-        dist = match.string_dist(u'My Song (EP)', u'My Song')
+        dist = string_dist(u'My Song (EP)', u'My Song')
         self.assertEqual(dist, 0.0)
 
     def test_featured_has_lower_weight(self):
-        dist1 = match.string_dist(u'My Song blah Someone', u'My Song')
-        dist2 = match.string_dist(u'My Song feat Someone', u'My Song')
+        dist1 = string_dist(u'My Song blah Someone', u'My Song')
+        dist2 = string_dist(u'My Song feat Someone', u'My Song')
         self.assert_(dist2 < dist1)
 
     def test_postfix_the(self):
-        dist = match.string_dist(u'The Song Title', u'Song Title, The')
+        dist = string_dist(u'The Song Title', u'Song Title, The')
         self.assertEqual(dist, 0.0)
 
     def test_postfix_a(self):
-        dist = match.string_dist(u'A Song Title', u'Song Title, A')
+        dist = string_dist(u'A Song Title', u'Song Title, A')
         self.assertEqual(dist, 0.0)
 
     def test_postfix_an(self):
-        dist = match.string_dist(u'An Album Title', u'Album Title, An')
+        dist = string_dist(u'An Album Title', u'Album Title, An')
         self.assertEqual(dist, 0.0)
 
     def test_empty_strings(self):
-        dist = match.string_dist(u'', u'')
+        dist = string_dist(u'', u'')
         self.assertEqual(dist, 0.0)
 
     def test_solo_pattern(self):
         # Just make sure these don't crash.
-        match.string_dist(u'The ', u'')
-        match.string_dist(u'(EP)', u'(EP)')
-        match.string_dist(u', An', u'')
+        string_dist(u'The ', u'')
+        string_dist(u'(EP)', u'(EP)')
+        string_dist(u', An', u'')
 
     def test_heuristic_does_not_harm_distance(self):
-        dist = match.string_dist(u'Untitled', u'[Untitled]')
+        dist = string_dist(u'Untitled', u'[Untitled]')
         self.assertEqual(dist, 0.0)
 
     def test_ampersand_expansion(self):
-        dist = match.string_dist(u'And', u'&')
+        dist = string_dist(u'And', u'&')
         self.assertEqual(dist, 0.0)
 
     def test_accented_characters(self):
-        dist = match.string_dist(u'\xe9\xe1\xf1', u'ean')
+        dist = string_dist(u'\xe9\xe1\xf1', u'ean')
         self.assertEqual(dist, 0.0)
 
 def suite():
