@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2012, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -24,31 +24,24 @@ from beets import library
 from beets import importer
 from beets import mediafile
 from beets.autotag import AlbumInfo, TrackInfo, AlbumMatch, TrackMatch
+from beets import config
 
 TEST_TITLES = ('The Opener', 'The Second Track', 'The Last Track')
-class NonAutotaggedImportTest(unittest.TestCase):
+class NonAutotaggedImportTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(NonAutotaggedImportTest, self).setUp()
+
         self.io.install()
 
-        self.libdb = os.path.join(_common.RSRC, 'testlib.blb')
+        self.libdb = os.path.join(self.temp_dir, 'testlib.blb')
         self.lib = library.Library(self.libdb)
-        self.libdir = os.path.join(_common.RSRC, 'testlibdir')
+        self.libdir = os.path.join(self.temp_dir, 'testlibdir')
         self.lib.directory = self.libdir
         self.lib.path_formats = [(
             'default', os.path.join('$artist', '$album', '$title')
         )]
 
-        self.srcdir = os.path.join(_common.RSRC, 'testsrcdir')
-
-    def tearDown(self):
-        self.io.restore()
-        if os.path.exists(self.libdb):
-            os.remove(self.libdb)
-        if os.path.exists(self.libdir):
-            shutil.rmtree(self.libdir)
-        if os.path.exists(self.srcdir):
-            shutil.rmtree(self.srcdir)
+        self.srcdir = os.path.join(self.temp_dir, 'testsrcdir')
 
     def _create_test_file(self, filepath, metadata):
         """Creates an mp3 file at the given path within self.srcdir.
@@ -83,31 +76,16 @@ class NonAutotaggedImportTest(unittest.TestCase):
                 }))
 
         # Run the UI "beet import" command!
-        importer.run_import(
-            lib=self.lib,
-            paths=[os.path.dirname(paths[0])],
-            copy=not move,
-            move=move,
-            write=True,
-            autot=False,
-            logfile=None,
-            threaded=threaded,
-            color=False,
-            delete=delete,
-            quiet=True,
-            resume=False,
-            quiet_fallback='skip',
-            choose_match_func = None,
-            should_resume_func = None,
-            singletons = singletons,
-            choose_item_func = None,
-            timid = False,
-            query = None,
-            incremental = False,
-            ignore = [],
-            resolve_duplicate_func = None,
-            per_disc_numbering = False,
-        )
+        config['import']['delete'] = delete
+        config['import']['threaded'] = threaded
+        config['import']['singletons'] = singletons
+        config['import']['move'] = move
+        config['import']['autotag'] = False
+        session = importer.ImportSession(self.lib,
+                                            logfile=None,
+                                            paths=[os.path.dirname(paths[0])],
+                                            query=None)
+        session.run()
 
         return paths
 
@@ -168,7 +146,7 @@ class NonAutotaggedImportTest(unittest.TestCase):
 
 # Utilities for invoking the apply_choices, manipulate_files, and finalize
 # coroutines.
-def _call_stages(config, items, choice_or_info,
+def _call_stages(session, items, choice_or_info,
                  stages=[importer.apply_choices,
                          importer.manipulate_files,
                          importer.finalize],
@@ -187,25 +165,28 @@ def _call_stages(config, items, choice_or_info,
 
     # Call the coroutines.
     for stage in stages:
-        coro = stage(config)
+        coro = stage(session)
         coro.next()
         coro.send(task)
 
     return task
 
-class ImportApplyTest(unittest.TestCase, _common.ExtraAsserts):
+class ImportApplyTest(_common.TestCase):
     def setUp(self):
-        self.libdir = os.path.join(_common.RSRC, 'testlibdir')
+        super(ImportApplyTest, self).setUp()
+
+        self.libdir = os.path.join(self.temp_dir, 'testlibdir')
         os.mkdir(self.libdir)
-        self.libpath = os.path.join(_common.RSRC, 'testlib.blb')
+        self.libpath = os.path.join(self.temp_dir, 'testlib.blb')
         self.lib = library.Library(self.libpath, self.libdir)
         self.lib.path_formats = [
             ('default', 'one'),
             ('singleton:true', 'three'),
             ('comp:true', 'two'),
         ]
+        self.session = _common.import_session(self.lib)
 
-        self.srcdir = os.path.join(_common.RSRC, 'testsrcdir')
+        self.srcdir = os.path.join(self.temp_dir, 'testsrcdir')
         os.mkdir(self.srcdir)
         os.mkdir(os.path.join(self.srcdir, 'testalbum'))
         self.srcpath = os.path.join(self.srcdir, 'testalbum', 'srcfile.mp3')
@@ -225,44 +206,34 @@ class ImportApplyTest(unittest.TestCase, _common.ExtraAsserts):
             albumtype = 'soundtrack',
         )
 
-    def tearDown(self):
-        shutil.rmtree(self.libdir)
-        if os.path.exists(self.srcdir):
-            shutil.rmtree(self.srcdir)
-        if os.path.exists(self.libpath):
-            os.unlink(self.libpath)
-
     def test_finalize_no_delete(self):
-        config = _common.iconfig(self.lib, delete=False)
-        _call_stages(config, [self.i], self.info)
+        config['import']['delete'] = False
+        _call_stages(self.session, [self.i], self.info)
         self.assertExists(self.srcpath)
 
     def test_finalize_with_delete(self):
-        config = _common.iconfig(self.lib, delete=True)
-        _call_stages(config, [self.i], self.info)
+        config['import']['delete'] = True
+        _call_stages(self.session, [self.i], self.info)
         self.assertNotExists(self.srcpath)
 
     def test_finalize_with_delete_prunes_directory_empty(self):
-        config = _common.iconfig(self.lib, delete=True)
-        _call_stages(config, [self.i], self.info,
+        config['import']['delete'] = True
+        _call_stages(self.session, [self.i], self.info,
                      toppath=self.srcdir)
         self.assertNotExists(os.path.dirname(self.srcpath))
 
     def test_apply_asis_uses_album_path(self):
-        config = _common.iconfig(self.lib)
-        _call_stages(config, [self.i], importer.action.ASIS)
+        _call_stages(self.session, [self.i], importer.action.ASIS)
         self.assertExists(os.path.join(self.libdir, 'one.mp3'))
 
     def test_apply_match_uses_album_path(self):
-        config = _common.iconfig(self.lib)
-        _call_stages(config, [self.i], self.info)
+        _call_stages(self.session, [self.i], self.info)
         self.assertExists(os.path.join(self.libdir, 'one.mp3'))
 
     def test_apply_tracks_uses_singleton_path(self):
-        config = _common.iconfig(self.lib)
-        apply_coro = importer.apply_choices(config)
+        apply_coro = importer.apply_choices(self.session)
         apply_coro.next()
-        manip_coro = importer.manipulate_files(config)
+        manip_coro = importer.manipulate_files(self.session)
         manip_coro.next()
 
         task = importer.ImportTask.item_task(self.i)
@@ -275,14 +246,13 @@ class ImportApplyTest(unittest.TestCase, _common.ExtraAsserts):
         )
 
     def test_apply_sentinel(self):
-        coro = importer.apply_choices(_common.iconfig(self.lib))
+        coro = importer.apply_choices(self.session)
         coro.next()
         coro.send(importer.ImportTask.done_sentinel('toppath'))
         # Just test no exception for now.
 
     def test_apply_populates_old_paths(self):
-        config = _common.iconfig(self.lib)
-        task = _call_stages(config, [self.i], self.info)
+        task = _call_stages(self.session, [self.i], self.info)
         self.assertEqual(task.old_paths, [self.srcpath])
 
     def test_reimport_inside_file_moves_and_does_not_add_to_old_paths(self):
@@ -300,8 +270,7 @@ class ImportApplyTest(unittest.TestCase, _common.ExtraAsserts):
         self.i.comp = False
 
         # Then, re-import the same file.
-        config =_common.iconfig(self.lib)
-        task = _call_stages(config, [self.i], self.info)
+        task = _call_stages(self.session, [self.i], self.info)
 
         # Old file should be gone.
         self.assertNotExists(internal_srcpath)
@@ -321,8 +290,7 @@ class ImportApplyTest(unittest.TestCase, _common.ExtraAsserts):
         self.lib._connection().commit()
 
         # Then, re-import the same file.
-        config = _common.iconfig(self.lib)
-        task = _call_stages(config, [self.i], self.info)
+        task = _call_stages(self.session, [self.i], self.info)
 
         # Old file should still exist.
         self.assertExists(self.srcpath)
@@ -333,31 +301,41 @@ class ImportApplyTest(unittest.TestCase, _common.ExtraAsserts):
         self.assertEqual(task.old_paths, [self.srcpath])
 
     def test_apply_with_move(self):
-        config = _common.iconfig(self.lib, move=True)
-        _call_stages(config, [self.i], self.info)
+        config['import']['move'] = True
+        _call_stages(self.session, [self.i], self.info)
         self.assertExists(list(self.lib.items())[0].path)
         self.assertNotExists(self.srcpath)
 
     def test_apply_with_move_prunes_empty_directory(self):
-        config = _common.iconfig(self.lib, move=True)
-        _call_stages(config, [self.i], self.info, toppath=self.srcdir)
+        config['import']['move'] = True
+        _call_stages(self.session, [self.i], self.info, toppath=self.srcdir)
+        self.assertNotExists(os.path.dirname(self.srcpath))
+
+    def test_apply_with_move_prunes_with_extra_clutter(self):
+        f = open(os.path.join(self.srcdir, 'testalbum', 'alog.log'), 'w')
+        f.close()
+        config['clutter'] = ['*.log']
+        config['import']['move'] = True
+        _call_stages(self.session, [self.i], self.info, toppath=self.srcdir)
         self.assertNotExists(os.path.dirname(self.srcpath))
 
     def test_manipulate_files_with_null_move(self):
         """It should be possible to "move" a file even when the file is
         already at the destination.
         """
-        config = _common.iconfig(self.lib, move=True)
         self.lib.move(self.i)  # Already at destination.
-        _call_stages(config, [self.i], self.info, toppath=self.srcdir,
+        config['import']['move'] = True
+        _call_stages(self.session, [self.i], self.info, toppath=self.srcdir,
                      stages=[importer.manipulate_files])
         self.assertExists(self.i.path)
 
-class AsIsApplyTest(unittest.TestCase):
+class AsIsApplyTest(_common.TestCase):
     def setUp(self):
-        self.dbpath = os.path.join(_common.RSRC, 'templib.blb')
+        super(AsIsApplyTest, self).setUp()
+
+        self.dbpath = os.path.join(self.temp_dir, 'templib.blb')
         self.lib = library.Library(self.dbpath)
-        self.config = _common.iconfig(self.lib, write=False, copy=False)
+        self.session = _common.import_session(self.lib)
 
         # Make an "album" that has a homogenous artist. (Modified by
         # individual tests.)
@@ -371,12 +349,9 @@ class AsIsApplyTest(unittest.TestCase):
         i1.albumartist = i2.albumartist = i3.albumartist = ''
         self.items = [i1, i2, i3]
 
-    def tearDown(self):
-        os.remove(self.dbpath)
-
     def _apply_result(self):
         """Run the "apply" coroutines and get the resulting Album."""
-        _call_stages(self.config, self.items, importer.action.ASIS,
+        _call_stages(self.session, self.items, importer.action.ASIS,
                      stages=[importer.apply_choices])
         return self.lib.albums()[0]
 
@@ -398,30 +373,31 @@ class AsIsApplyTest(unittest.TestCase):
         self.assertFalse(alb.comp)
         self.assertEqual(alb.albumartist, self.items[2].artist)
 
-class ApplyExistingItemsTest(unittest.TestCase, _common.ExtraAsserts):
+class ApplyExistingItemsTest(_common.TestCase):
     def setUp(self):
-        self.libdir = os.path.join(_common.RSRC, 'testlibdir')
+        super(ApplyExistingItemsTest, self).setUp()
+
+        self.libdir = os.path.join(self.temp_dir, 'testlibdir')
         os.mkdir(self.libdir)
 
-        self.dbpath = os.path.join(_common.RSRC, 'templib.blb')
+        self.dbpath = os.path.join(self.temp_dir, 'templib.blb')
         self.lib = library.Library(self.dbpath, self.libdir)
         self.lib.path_formats = [
             ('default', '$artist/$title'),
         ]
-        self.config = _common.iconfig(self.lib, write=False, copy=False)
+        self.session = _common.import_session(self.lib)
+
+        config['import']['write'] = False
+        config['import']['copy'] = False
 
         self.srcpath = os.path.join(self.libdir, 'srcfile.mp3')
         shutil.copy(os.path.join(_common.RSRC, 'full.mp3'), self.srcpath)
         self.i = library.Item.from_path(self.srcpath)
         self.i.comp = False
 
-    def tearDown(self):
-        os.remove(self.dbpath)
-        shutil.rmtree(self.libdir)
-
     def _apply_asis(self, items, album=True):
         """Run the "apply" coroutine."""
-        _call_stages(self.config, items, importer.action.ASIS, album=album,
+        _call_stages(self.session, items, importer.action.ASIS, album=album,
                      stages=[importer.apply_choices,
                              importer.manipulate_files])
 
@@ -458,7 +434,7 @@ class ApplyExistingItemsTest(unittest.TestCase, _common.ExtraAsserts):
 
     def test_apply_existing_item_new_metadata_does_not_duplicate(self):
         # We want to copy the item to a new location.
-        self.config.copy = True
+        config['import']['copy'] = True
 
         # Import with existing metadata.
         self._apply_asis([self.i])
@@ -475,7 +451,8 @@ class ApplyExistingItemsTest(unittest.TestCase, _common.ExtraAsserts):
 
     def test_apply_existing_item_new_metadata_moves_files(self):
         # As above, import with old metadata and then reimport with new.
-        self.config.copy = True
+        config['import']['copy'] = True
+
         self._apply_asis([self.i])
         item = self.lib.items().next()
         new_item = library.Item.from_path(item.path)
@@ -488,7 +465,8 @@ class ApplyExistingItemsTest(unittest.TestCase, _common.ExtraAsserts):
 
     def test_apply_existing_item_new_metadata_copy_disabled(self):
         # Import *without* copying to ensure that the path does *not* change.
-        self.config.copy = False
+        config['import']['copy'] = False
+
         self._apply_asis([self.i])
         item = self.lib.items().next()
         new_item = library.Item.from_path(item.path)
@@ -500,7 +478,8 @@ class ApplyExistingItemsTest(unittest.TestCase, _common.ExtraAsserts):
         self.assertExists(item.path)
 
     def test_apply_existing_item_new_metadata_removes_old_files(self):
-        self.config.copy = True
+        config['import']['copy'] = True
+
         self._apply_asis([self.i])
         item = self.lib.items().next()
         oldpath = item.path
@@ -514,8 +493,9 @@ class ApplyExistingItemsTest(unittest.TestCase, _common.ExtraAsserts):
     def test_apply_existing_item_new_metadata_delete_enabled(self):
         # The "delete" flag should be ignored -- only the "copy" flag
         # controls whether files move.
-        self.config.copy = True
-        self.config.delete = True # !
+        config['import']['copy'] = True
+        config['import']['delete'] = True  # !
+
         self._apply_asis([self.i])
         item = self.lib.items().next()
         oldpath = item.path
@@ -530,7 +510,8 @@ class ApplyExistingItemsTest(unittest.TestCase, _common.ExtraAsserts):
 
     def test_apply_existing_item_preserves_file(self):
         # With copying enabled, import the item twice with same metadata.
-        self.config.copy = True
+        config['import']['copy'] = True
+
         self._apply_asis([self.i])
         item = self.lib.items().next()
         oldpath = item.path
@@ -543,8 +524,9 @@ class ApplyExistingItemsTest(unittest.TestCase, _common.ExtraAsserts):
         self.assertExists(oldpath)
 
     def test_apply_existing_item_preserves_file_delete_enabled(self):
-        self.config.copy = True
-        self.config.delete = True # !
+        config['import']['copy'] = True
+        config['import']['delete'] = True  # !
+
         self._apply_asis([self.i])
         item = self.lib.items().next()
         new_item = library.Item.from_path(item.path)
@@ -565,8 +547,10 @@ class ApplyExistingItemsTest(unittest.TestCase, _common.ExtraAsserts):
         self.assertEqual(len(list(self.lib.items())), 1)
         self.assertEqual(len(list(self.lib.albums())), 1)
 
-class InferAlbumDataTest(unittest.TestCase):
+class InferAlbumDataTest(_common.TestCase):
     def setUp(self):
+        super(InferAlbumDataTest, self).setUp()
+
         i1 = _common.item()
         i2 = _common.item()
         i3 = _common.item()
@@ -578,7 +562,7 @@ class InferAlbumDataTest(unittest.TestCase):
         i1.mb_albumartistid = i2.mb_albumartistid = i3.mb_albumartistid = ''
         self.items = [i1, i2, i3]
 
-        self.task = importer.ImportTask(path='a path', toppath='top path',
+        self.task = importer.ImportTask(paths=['a path'], toppath='top path',
                                         items=self.items)
         self.task.set_null_candidates()
 
@@ -657,8 +641,10 @@ class InferAlbumDataTest(unittest.TestCase):
         self.assertFalse(self.items[1].comp)
         self.assertEqual(self.items[1].albumartist, self.items[2].artist)
 
-class DuplicateCheckTest(unittest.TestCase):
+class DuplicateCheckTest(_common.TestCase):
     def setUp(self):
+        super(DuplicateCheckTest, self).setUp()
+
         self.lib = library.Library(':memory:')
         self.i = _common.item()
         self.album = self.lib.add_album([self.i])
@@ -671,7 +657,7 @@ class DuplicateCheckTest(unittest.TestCase):
         artist = artist or item.albumartist
         album = album or item.album
 
-        task = importer.ImportTask(path='a path', toppath='top path',
+        task = importer.ImportTask(paths=['a path'], toppath='top path',
                                    items=[item])
         task.set_candidates(artist, album, None, None)
         if asis:
@@ -752,15 +738,17 @@ class DuplicateCheckTest(unittest.TestCase):
                                         self._item_task(False, existing=True))
         self.assertFalse(res)
 
-class TagLogTest(unittest.TestCase):
+class TagLogTest(_common.TestCase):
     def test_tag_log_line(self):
         sio = StringIO.StringIO()
-        importer.tag_log(sio, 'status', 'path')
+        session = _common.import_session(logfile=sio)
+        session.tag_log('status', 'path')
         assert 'status path' in sio.getvalue()
 
     def test_tag_log_unicode(self):
         sio = StringIO.StringIO()
-        importer.tag_log(sio, 'status', 'caf\xc3\xa9')
+        session = _common.import_session(logfile=sio)
+        session.tag_log('status', 'caf\xc3\xa9')
         assert 'status caf' in sio.getvalue()
 
 def suite():
