@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2012, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -14,20 +14,23 @@
 
 """Tests for MusicBrainz API wrapper.
 """
+import _common
 from _common import unittest
 from beets.autotag import mb
+from beets import config
 
-class MBAlbumInfoTest(unittest.TestCase):
+class MBAlbumInfoTest(_common.TestCase):
     def _make_release(self, date_str='2009', tracks=None):
         release = {
             'title': 'ALBUM TITLE',
             'id': 'ALBUM ID',
             'asin': 'ALBUM ASIN',
+            'disambiguation': 'R_DISAMBIGUATION',
             'release-group': {
                 'type': 'Album',
                 'first-release-date': date_str,
                 'id': 'RELEASE GROUP ID',
-                'disambiguation': 'DISAMBIGUATION',
+                'disambiguation': 'RG_DISAMBIGUATION',
             },
             'artist-credit': [
                 {
@@ -94,7 +97,8 @@ class MBAlbumInfoTest(unittest.TestCase):
         self.assertEqual(d.album_id, 'ALBUM ID')
         self.assertEqual(d.artist, 'ARTIST NAME')
         self.assertEqual(d.artist_id, 'ARTIST ID')
-        self.assertEqual(d.year, 1984)
+        self.assertEqual(d.original_year, 1984)
+        self.assertEqual(d.year, 3001)
         self.assertEqual(d.artist_credit, 'ARTIST CREDIT')
 
     def test_parse_release_type(self):
@@ -105,9 +109,9 @@ class MBAlbumInfoTest(unittest.TestCase):
     def test_parse_release_full_date(self):
         release = self._make_release('1987-03-31')
         d = mb.album_info(release)
-        self.assertEqual(d.year, 1987)
-        self.assertEqual(d.month, 3)
-        self.assertEqual(d.day, 31)
+        self.assertEqual(d.original_year, 1987)
+        self.assertEqual(d.original_month, 3)
+        self.assertEqual(d.original_day, 31)
 
     def test_parse_tracks(self):
         tracks = [self._make_track('TITLE ONE', 'ID ONE', 100.0 * 1000.0),
@@ -173,8 +177,8 @@ class MBAlbumInfoTest(unittest.TestCase):
     def test_parse_release_year_month_only(self):
         release = self._make_release('1987-03')
         d = mb.album_info(release)
-        self.assertEqual(d.year, 1987)
-        self.assertEqual(d.month, 3)
+        self.assertEqual(d.original_year, 1987)
+        self.assertEqual(d.original_month, 3)
 
     def test_no_durations(self):
         tracks = [self._make_track('TITLE', 'ID', None)]
@@ -185,9 +189,9 @@ class MBAlbumInfoTest(unittest.TestCase):
     def test_no_release_date(self):
         release = self._make_release(None)
         d = mb.album_info(release)
-        self.assertFalse(d.year)
-        self.assertFalse(d.month)
-        self.assertFalse(d.day)
+        self.assertFalse(d.original_year)
+        self.assertFalse(d.original_month)
+        self.assertFalse(d.original_day)
 
     def test_various_artists_defaults_false(self):
         release = self._make_release(None)
@@ -247,7 +251,8 @@ class MBAlbumInfoTest(unittest.TestCase):
     def test_parse_disambig(self):
         release = self._make_release(None)
         d = mb.album_info(release)
-        self.assertEqual(d.albumdisambig, 'DISAMBIGUATION')
+        self.assertEqual(d.albumdisambig,
+                         'RG_DISAMBIGUATION, R_DISAMBIGUATION')
 
     def test_parse_disctitle(self):
         tracks = [self._make_track('TITLE ONE', 'ID ONE', 100.0 * 1000.0),
@@ -273,7 +278,24 @@ class MBAlbumInfoTest(unittest.TestCase):
         self.assertEqual(track.artist_sort, 'TRACK ARTIST SORT NAME')
         self.assertEqual(track.artist_credit, 'TRACK ARTIST CREDIT')
 
-class ArtistFlatteningTest(unittest.TestCase):
+class ParseIDTest(_common.TestCase):
+    def test_parse_id_correct(self):
+        id_string = "28e32c71-1450-463e-92bf-e0a46446fc11"
+        out = mb._parse_id(id_string)
+        self.assertEqual(out, id_string)
+
+    def test_parse_id_non_id_returns_none(self):
+        id_string = "blah blah"
+        out = mb._parse_id(id_string)
+        self.assertEqual(out, None)
+
+    def test_parse_id_url_finds_id(self):
+        id_string = "28e32c71-1450-463e-92bf-e0a46446fc11"
+        id_url = "http://musicbrainz.org/entity/%s" % id_string
+        out = mb._parse_id(id_url)
+        self.assertEqual(out, id_string)
+
+class ArtistFlatteningTest(_common.TestCase):
     def _credit_dict(self, suffix=''):
         return {
             'artist': {
@@ -283,6 +305,18 @@ class ArtistFlatteningTest(unittest.TestCase):
             'name': 'CREDIT' + suffix,
         }
 
+    def _add_alias(self, credit_dict, suffix='', locale='', primary=False):
+        alias = {
+            'alias': 'ALIAS' + suffix,
+            'locale': locale,
+            'sort-name': 'ALIASSORT' + suffix
+        }
+        if primary:
+            alias['primary'] = 'primary'
+        if 'alias-list' not in credit_dict['artist']:
+            credit_dict['artist']['alias-list'] = []
+        credit_dict['artist']['alias-list'].append(alias)
+
     def test_single_artist(self):
         a, s, c = mb._flatten_artist_credit([self._credit_dict()])
         self.assertEqual(a, 'NAME')
@@ -297,6 +331,38 @@ class ArtistFlatteningTest(unittest.TestCase):
         self.assertEqual(s, 'SORTa AND SORTb')
         self.assertEqual(c, 'CREDITa AND CREDITb')
 
+    def test_alias(self):
+        credit_dict = self._credit_dict()
+        self._add_alias(credit_dict, suffix='en', locale='en')
+        self._add_alias(credit_dict, suffix='en_GB', locale='en_GB')
+        self._add_alias(credit_dict, suffix='fr', locale='fr')
+        self._add_alias(credit_dict, suffix='fr_P', locale='fr', primary=True)
+
+        # test no alias
+        config['import']['languages'] = ['']
+        flat = mb._flatten_artist_credit([credit_dict])
+        self.assertEqual(flat, ('NAME', 'SORT', 'CREDIT'))
+
+        # test en
+        config['import']['languages'] = ['en']
+        flat = mb._flatten_artist_credit([credit_dict])
+        self.assertEqual(flat, ('ALIASen', 'ALIASSORTen', 'CREDIT'))
+
+        # test en_GB en
+        config['import']['languages'] = ['en_GB', 'en']
+        flat = mb._flatten_artist_credit([credit_dict])
+        self.assertEqual(flat, ('ALIASen_GB', 'ALIASSORTen_GB', 'CREDIT'))
+
+        # test en en_GB
+        config['import']['languages'] = ['en', 'en_GB']
+        flat = mb._flatten_artist_credit([credit_dict])
+        self.assertEqual(flat, ('ALIASen', 'ALIASSORTen', 'CREDIT'))
+
+        # test fr primary
+        config['import']['languages'] = ['fr']
+        flat = mb._flatten_artist_credit([credit_dict])
+        self.assertEqual(flat, ('ALIASfr_P', 'ALIASSORTfr_P', 'CREDIT'))
+
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
