@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2012, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -120,6 +120,10 @@ CORRECT_DICTS = {
         'albumdisambig':        u'',
         'artist_credit':        u'',
         'albumartist_credit':   u'',
+        'original_year':        0,
+        'original_month':       0,
+        'original_day':         0,
+        'original_date':        datetime.date.min,
     },
 
     # Full release date.
@@ -189,12 +193,30 @@ READ_ONLY_CORRECT_DICTS = {
 
     'full.mpc': {
         'length': 1.0,
-        'bitrate': 23,
+        'bitrate': 23458,
         'format': 'Musepack',
         'samplerate': 44100,
         'bitdepth': 0,
         'channels': 2,
     },
+
+    'full.wma': {
+        'length': 1.0,
+        'bitrate': 128000,
+        'format': 'Windows Media',
+        'samplerate': 44100,
+        'bitdepth': 0,
+        'channels': 1,
+    },
+
+    'full.alac.m4a': {
+        'length': 1.0,
+        'bitrate': 55072,
+        'format': 'ALAC',
+        'samplerate': 0,
+        'bitdepth': 0,
+        'channels': 0,
+    },
 }
 
 TEST_FILES = {
@@ -205,6 +227,7 @@ TEST_FILES = {
     'ape': ['full'],
     'wv': ['full'],
     'mpc': ['full'],
+    'wma': ['full'],
 }
 
 class AllFilesMixin(object):
@@ -250,6 +273,12 @@ class AllFilesMixin(object):
     def test_mpc(self):
         self._run('full', 'mpc')
 
+    def test_wma(self):
+        self._run('full', 'wma')
+
+    def test_alac(self):
+        self._run('full', 'alac.m4a')
+
     # Special test for advanced release date.
     def test_date_mp3(self):
         self._run('date', 'mp3')
@@ -409,6 +438,12 @@ class ReadOnlyTest(unittest.TestCase):
     def test_mpc(self):
         self._run('full.mpc')
 
+    def test_wma(self):
+        self._run('full.wma')
+
+    def test_alac(self):
+        self._run('full.alac.m4a')
+
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
 
