@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2011, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -20,82 +20,74 @@ import _common
 from _common import unittest
 import beets.library
 
-pqp = beets.library.CollectionQuery._parse_query_part
+pqp = beets.library.parse_query_part
 
 some_item = _common.item()
 
 class QueryParseTest(unittest.TestCase):
     def test_one_basic_term(self):
         q = 'test'
-        r = (None, 'test', False)
+        r = (None, 'test', beets.library.SubstringQuery)
         self.assertEqual(pqp(q), r)
 
     def test_one_keyed_term(self):
         q = 'test:val'
-        r = ('test', 'val', False)
+        r = ('test', 'val', beets.library.SubstringQuery)
         self.assertEqual(pqp(q), r)
 
     def test_colon_at_end(self):
         q = 'test:'
-        r = (None, 'test:', False)
+        r = (None, 'test:', beets.library.SubstringQuery)
         self.assertEqual(pqp(q), r)
 
     def test_one_basic_regexp(self):
         q = r':regexp'
-        r = (None, 'regexp', True)
+        r = (None, 'regexp', beets.library.RegexpQuery)
         self.assertEqual(pqp(q), r)
 
     def test_keyed_regexp(self):
         q = r'test::regexp'
-        r = ('test', 'regexp', True)
+        r = ('test', 'regexp', beets.library.RegexpQuery)
         self.assertEqual(pqp(q), r)
 
     def test_escaped_colon(self):
         q = r'test\:val'
-        r = (None, 'test:val', False)
+        r = (None, 'test:val', beets.library.SubstringQuery)
         self.assertEqual(pqp(q), r)
 
     def test_escaped_colon_in_regexp(self):
         q = r':test\:regexp'
-        r = (None, 'test:regexp', True)
+        r = (None, 'test:regexp', beets.library.RegexpQuery)
         self.assertEqual(pqp(q), r)
 
-class AnySubstringQueryTest(unittest.TestCase):
+    def test_single_year(self):
+        q = 'year:1999'
+        r = ('year', '1999', beets.library.NumericQuery)
+        self.assertEqual(pqp(q), r)
+
+    def test_multiple_years(self):
+        q = 'year:1999..2010'
+        r = ('year', '1999..2010', beets.library.NumericQuery)
+        self.assertEqual(pqp(q), r)
+
+class AnyFieldQueryTest(unittest.TestCase):
     def setUp(self):
         self.lib = beets.library.Library(':memory:')
         self.lib.add(some_item)
 
     def test_no_restriction(self):
-        q = beets.library.AnySubstringQuery('title')
+        q = beets.library.AnyFieldQuery('title', beets.library.ITEM_KEYS,
+                                        beets.library.SubstringQuery)
         self.assertEqual(self.lib.items(q).next().title, 'the title')
 
     def test_restriction_completeness(self):
-        q = beets.library.AnySubstringQuery('title', ['title'])
+        q = beets.library.AnyFieldQuery('title', ['title'],
+                                        beets.library.SubstringQuery)
         self.assertEqual(self.lib.items(q).next().title, 'the title')
 
     def test_restriction_soundness(self):
-        q = beets.library.AnySubstringQuery('title', ['artist'])
-        self.assertRaises(StopIteration, self.lib.items(q).next)
-
-class AnyRegexpQueryTest(unittest.TestCase):
-    def setUp(self):
-        self.lib = beets.library.Library(':memory:')
-        self.lib.add(some_item)
-
-    def test_no_restriction(self):
-        q = beets.library.AnyRegexpQuery(r'^the ti')
-        self.assertEqual(self.lib.items(q).next().title, 'the title')
-
-    def test_restriction_completeness(self):
-        q = beets.library.AnyRegexpQuery(r'^the ti', ['title'])
-        self.assertEqual(self.lib.items(q).next().title, 'the title')
-
-    def test_restriction_soundness(self):
-        q = beets.library.AnyRegexpQuery(r'^the ti', ['artist'])
-        self.assertRaises(StopIteration, self.lib.items(q).next)
-
-    def test_restriction_soundness_2(self):
-        q = beets.library.AnyRegexpQuery(r'the ti$', ['title'])
+        q = beets.library.AnyFieldQuery('title', ['artist'],
+                                        beets.library.SubstringQuery)
         self.assertRaises(StopIteration, self.lib.items(q).next)
 
 # Convenient asserts for matching items.
@@ -237,6 +229,25 @@ class GetTest(unittest.TestCase, AssertsMixin):
         self.assert_matched(results, 'Littlest Things')
         self.assert_done(results)
 
+    def test_single_year(self):
+        q = 'year:2006'
+        results = self.lib.items(q)
+        self.assert_matched(results, 'Littlest Things')
+        self.assert_matched(results, 'Lovers Who Uncover')
+        self.assert_done(results)
+
+    def test_year_range(self):
+        q = 'year:2000..2010'
+        results = self.lib.items(q)
+        self.assert_matched(results, 'Littlest Things')
+        self.assert_matched(results, 'Take Pills')
+        self.assert_matched(results, 'Lovers Who Uncover')
+        self.assert_done(results)
+
+    def test_bad_year(self):
+        q = 'year:delete from items'
+        self.assertRaises(ValueError, self.lib.items, q)
+
 class MemoryGetTest(unittest.TestCase, AssertsMixin):
     def setUp(self):
         self.album_item = _common.item()
@@ -330,6 +341,22 @@ class MatchTest(unittest.TestCase):
         q = beets.library.SubstringQuery('disc', '6')
         self.assertTrue(q.match(self.item))
 
+    def test_year_match_positive(self):
+        q = beets.library.NumericQuery('year', '1')
+        self.assertTrue(q.match(self.item))
+
+    def test_year_match_negative(self):
+        q = beets.library.NumericQuery('year', '10')
+        self.assertFalse(q.match(self.item))
+
+    def test_bitrate_range_positive(self):
+        q = beets.library.NumericQuery('bitrate', '100000..200000')
+        self.assertTrue(q.match(self.item))
+
+    def test_bitrate_range_negative(self):
+        q = beets.library.NumericQuery('bitrate', '200000..300000')
+        self.assertFalse(q.match(self.item))
+
 class PathQueryTest(unittest.TestCase, AssertsMixin):
     def setUp(self):
         self.lib = beets.library.Library(':memory:')
@@ -428,8 +455,6 @@ class BrowseTest(unittest.TestCase, AssertsMixin):
         items = self.lib.items('2007')
         self.assert_done(items)
 
-    #FIXME Haven't tested explicit (non-query) criteria.
-
 class CountTest(unittest.TestCase):
     def setUp(self):
         self.lib = beets.library.Library(':memory:')
