@@ -1,5 +1,5 @@
 # This file is part of beets.
-# Copyright 2011, Adrian Sampson.
+# Copyright 2013, Adrian Sampson.
 #
 # Permission is hereby granted, free of charge, to any person obtaining
 # a copy of this software and associated documentation files (the
@@ -19,8 +19,7 @@ import shutil
 import textwrap
 import logging
 import re
-from StringIO import StringIO
-import ConfigParser
+import yaml
 
 import _common
 from _common import unittest
@@ -28,12 +27,15 @@ from beets import library
 from beets import ui
 from beets.ui import commands
 from beets import autotag
+from beets.autotag.match import distance
 from beets import importer
 from beets.mediafile import MediaFile
+from beets import config
+from beets.util import confit
 
-class ListTest(unittest.TestCase):
+class ListTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(ListTest, self).setUp()
         self.io.install()
 
         self.lib = library.Library(':memory:')
@@ -43,11 +45,8 @@ class ListTest(unittest.TestCase):
         self.lib.add_album([i])
         self.item = i
 
-    def tearDown(self):
-        self.io.restore()
-
     def _run_list(self, query='', album=False, path=False, fmt=None):
-        commands.list_items(self.lib, query, album, fmt, None)
+        commands.list_items(self.lib, query, album, fmt)
 
     def test_list_outputs_item(self):
         self._run_list()
@@ -114,12 +113,13 @@ class ListTest(unittest.TestCase):
         self.assertTrue(u'the genre' in out)
         self.assertTrue(u'the album' not in out)
 
-class RemoveTest(unittest.TestCase):
+class RemoveTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(RemoveTest, self).setUp()
+
         self.io.install()
 
-        self.libdir = os.path.join(_common.RSRC, 'testlibdir')
+        self.libdir = os.path.join(self.temp_dir, 'testlibdir')
         os.mkdir(self.libdir)
 
         # Copy a file into the library.
@@ -127,31 +127,27 @@ class RemoveTest(unittest.TestCase):
         self.i = library.Item.from_path(os.path.join(_common.RSRC, 'full.mp3'))
         self.lib.add(self.i, True)
 
-    def tearDown(self):
-        self.io.restore()
-        shutil.rmtree(self.libdir)
-
     def test_remove_items_no_delete(self):
         self.io.addinput('y')
-        commands.remove_items(self.lib, '', False, False, None)
+        commands.remove_items(self.lib, '', False, False)
         items = self.lib.items()
         self.assertEqual(len(list(items)), 0)
         self.assertTrue(os.path.exists(self.i.path))
 
     def test_remove_items_with_delete(self):
         self.io.addinput('y')
-        commands.remove_items(self.lib, '', False, True, None)
+        commands.remove_items(self.lib, '', False, True)
         items = self.lib.items()
         self.assertEqual(len(list(items)), 0)
         self.assertFalse(os.path.exists(self.i.path))
 
-class ModifyTest(unittest.TestCase):
+class ModifyTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(ModifyTest, self).setUp()
+
         self.io.install()
 
-        self.libdir = os.path.join(_common.RSRC, 'testlibdir')
-        os.mkdir(self.libdir)
+        self.libdir = os.path.join(self.temp_dir, 'testlibdir')
 
         # Copy a file into the library.
         self.lib = library.Library(':memory:', self.libdir)
@@ -159,14 +155,10 @@ class ModifyTest(unittest.TestCase):
         self.lib.add(self.i, True)
         self.album = self.lib.add_album([self.i])
 
-    def tearDown(self):
-        self.io.restore()
-        shutil.rmtree(self.libdir)
-
     def _modify(self, mods, query=(), write=False, move=False, album=False):
         self.io.addinput('y')
         commands.modify_items(self.lib, mods, query,
-                              write, move, album, True, True, None)
+                              write, move, album, True)
 
     def test_modify_item_dbdata(self):
         self._modify(["title=newTitle"])
@@ -224,12 +216,13 @@ class ModifyTest(unittest.TestCase):
         item.read()
         self.assertFalse('newAlbum' in item.path)
 
-class MoveTest(unittest.TestCase, _common.ExtraAsserts):
+class MoveTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(MoveTest, self).setUp()
+
         self.io.install()
 
-        self.libdir = os.path.join(_common.RSRC, 'testlibdir')
+        self.libdir = os.path.join(self.temp_dir, 'testlibdir')
         os.mkdir(self.libdir)
 
         self.itempath = os.path.join(self.libdir, 'srcfile')
@@ -242,13 +235,7 @@ class MoveTest(unittest.TestCase, _common.ExtraAsserts):
         self.album = self.lib.add_album([self.i])
 
         # Alternate destination directory.
-        self.otherdir = os.path.join(_common.RSRC, 'testotherdir')
-
-    def tearDown(self):
-        self.io.restore()
-        shutil.rmtree(self.libdir)
-        if os.path.exists(self.otherdir):
-            shutil.rmtree(self.otherdir)
+        self.otherdir = os.path.join(self.temp_dir, 'testotherdir')
 
     def _move(self, query=(), dest=None, copy=False, album=False):
         commands.move_items(self.lib, dest, query, copy, album)
@@ -295,13 +282,13 @@ class MoveTest(unittest.TestCase, _common.ExtraAsserts):
         self.assertExists(self.i.path)
         self.assertNotExists(self.itempath)
 
-class UpdateTest(unittest.TestCase, _common.ExtraAsserts):
+class UpdateTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(UpdateTest, self).setUp()
+
         self.io.install()
 
-        self.libdir = os.path.join(_common.RSRC, 'testlibdir')
-        os.mkdir(self.libdir)
+        self.libdir = os.path.join(self.temp_dir, 'testlibdir')
 
         # Copy a file into the library.
         self.lib = library.Library(':memory:', self.libdir)
@@ -315,16 +302,12 @@ class UpdateTest(unittest.TestCase, _common.ExtraAsserts):
         self.album.set_art(artfile)
         os.remove(artfile)
 
-    def tearDown(self):
-        self.io.restore()
-        shutil.rmtree(self.libdir)
-
     def _update(self, query=(), album=False, move=False, reset_mtime=True):
         self.io.addinput('y')
         if reset_mtime:
             self.i.mtime = 0
             self.lib.store(self.i)
-        commands.update_items(self.lib, query, album, move, True, False, None)
+        commands.update_items(self.lib, query, album, move, False)
 
     def test_delete_removes_item(self):
         self.assertTrue(list(self.lib.items()))
@@ -399,12 +382,10 @@ class UpdateTest(unittest.TestCase, _common.ExtraAsserts):
         item = self.lib.items().next()
         self.assertEqual(item.title, 'full')
 
-class PrintTest(unittest.TestCase):
+class PrintTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(PrintTest, self).setUp()
         self.io.install()
-    def tearDown(self):
-        self.io.restore()
 
     def test_print_without_locale(self):
         lang = os.environ.get('LANG')
@@ -439,12 +420,10 @@ class PrintTest(unittest.TestCase):
             else:
                 del os.environ['LC_CTYPE']
 
-class AutotagTest(unittest.TestCase):
+class AutotagTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(AutotagTest, self).setUp()
         self.io.install()
-    def tearDown(self):
-        self.io.restore()
 
     def _no_candidates_test(self, result):
         task = importer.ImportTask(
@@ -452,8 +431,9 @@ class AutotagTest(unittest.TestCase):
             'path',
             [_common.item()],
         )
-        task.set_candidates('artist', 'album', [], autotag.RECOMMEND_NONE)
-        res = commands.choose_match(task, _common.iconfig(None, quiet=False))
+        task.set_candidates('artist', 'album', [], autotag.recommendation.none)
+        session = _common.import_session(cli=True)
+        res = session.choose_match(task)
         self.assertEqual(res, result)
         self.assertTrue('No match' in self.io.getoutput())
 
@@ -465,19 +445,17 @@ class AutotagTest(unittest.TestCase):
         self.io.addinput('u')
         self._no_candidates_test(importer.action.ASIS)
 
-class ImportTest(unittest.TestCase):
+class ImportTest(_common.TestCase):
     def test_quiet_timid_disallowed(self):
-        self.assertRaises(ui.UserError, commands.import_files,
-                          None, [], False, False, False, False, None,
-                          False, False, False, True, False, None, False, True,
-                          None, False, [], False)
+        config['import']['quiet'] = True
+        config['import']['timid'] = True
+        self.assertRaises(ui.UserError, commands.import_files, None, [],
+                          None)
 
-class InputTest(unittest.TestCase):
+class InputTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(InputTest, self).setUp()
         self.io.install()
-    def tearDown(self):
-        self.io.restore()
 
     def test_manual_search_gets_unicode(self):
         self.io.addinput('\xc3\x82me')
@@ -486,188 +464,154 @@ class InputTest(unittest.TestCase):
         self.assertEqual(artist, u'\xc2me')
         self.assertEqual(album, u'\xc2me')
 
-class ConfigTest(unittest.TestCase):
+class ConfigTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(ConfigTest, self).setUp()
         self.io.install()
         self.test_cmd = ui.Subcommand('test', help='test')
         commands.default_commands.append(self.test_cmd)
     def tearDown(self):
-        self.io.restore()
+        super(ConfigTest, self).tearDown()
         commands.default_commands.pop()
-    def _run_main(self, args, config, func):
+    def _run_main(self, args, config_yaml, func):
         self.test_cmd.func = func
-        ui._raw_main(args + ['test'], StringIO(config))
+        config_yaml = textwrap.dedent(config_yaml).strip()
+        if config_yaml:
+            config_data = yaml.load(config_yaml, Loader=confit.Loader)
+            config.set(config_data)
+        ui._raw_main(args + ['test'])
 
     def test_paths_section_respected(self):
-        def func(lib, config, opts, args):
+        def func(lib, opts, args):
             key, template = lib.path_formats[0]
             self.assertEqual(key, 'x')
             self.assertEqual(template.original, 'y')
-        self._run_main([], textwrap.dedent("""
-            [paths]
-            x=y"""), func)
+        self._run_main([], """
+            paths:
+                x: y
+        """, func)
 
     def test_default_paths_preserved(self):
-        def func(lib, config, opts, args):
+        default_formats = ui.get_path_formats()
+        def func(lib, opts, args):
             self.assertEqual(lib.path_formats[1:],
-                             ui.DEFAULT_PATH_FORMATS)
-        self._run_main([], textwrap.dedent("""
-            [paths]
-            x=y"""), func)
+                             default_formats)
+        self._run_main([], """
+            paths:
+                x: y
+        """, func)
 
     def test_nonexistant_config_file(self):
         os.environ['BEETSCONFIG'] = '/xxxxx'
         ui.main(['version'])
 
     def test_nonexistant_db(self):
-        def func(lib, config, opts, args):
+        def func(lib, opts, args):
             pass
         with self.assertRaises(ui.UserError):
-            self._run_main([], textwrap.dedent("""
-                [beets]
+            self._run_main([], """
                 library: /xxx/yyy/not/a/real/path
-            """), func)
+            """, func)
 
     def test_replacements_parsed(self):
-        def func(lib, config, opts, args):
+        def func(lib, opts, args):
             replacements = lib.replacements
             self.assertEqual(replacements, [(re.compile(ur'[xy]'), u'z')])
-        self._run_main([], textwrap.dedent("""
-            [beets]
-            replace=[xy] z"""), func)
-
-    def test_replacements_parsed_unicode(self):
-        def func(lib, config, opts, args):
-            replacements = lib.replacements
-            self.assertEqual(replacements, [(re.compile(ur'\u2019'), u'z')])
-        self._run_main([], textwrap.dedent(u"""
-            [beets]
-            replace=\u2019 z"""), func)
-
-    def test_empty_replacements_produce_none(self):
-        def func(lib, config, opts, args):
-            replacements = lib.replacements
-            self.assertFalse(replacements)
-        self._run_main([], textwrap.dedent("""
-            [beets]
-            """), func)
+        self._run_main([], """
+            replace:
+                '[xy]': z
+        """, func)
 
     def test_multiple_replacements_parsed(self):
-        def func(lib, config, opts, args):
+        def func(lib, opts, args):
             replacements = lib.replacements
             self.assertEqual(replacements, [
                 (re.compile(ur'[xy]'), u'z'),
                 (re.compile(ur'foo'), u'bar'),
             ])
-        self._run_main([], textwrap.dedent("""
-            [beets]
-            replace=[xy] z
-                foo bar"""), func)
+        self._run_main([], """
+            replace:
+                '[xy]': z
+                foo: bar
+        """, func)
 
-class ShowdiffTest(unittest.TestCase):
+class ShowdiffTest(_common.TestCase):
     def setUp(self):
-        self.io = _common.DummyIO()
+        super(ShowdiffTest, self).setUp()
         self.io.install()
-    def tearDown(self):
-        self.io.restore()
 
     def test_showdiff_strings(self):
-        commands._showdiff('field', 'old', 'new', True)
+        commands._showdiff('field', 'old', 'new')
         out = self.io.getoutput()
         self.assertTrue('field' in out)
 
     def test_showdiff_identical(self):
-        commands._showdiff('field', 'old', 'old', True)
+        commands._showdiff('field', 'old', 'old')
         out = self.io.getoutput()
         self.assertFalse('field' in out)
 
     def test_showdiff_ints(self):
-        commands._showdiff('field', 2, 3, True)
+        commands._showdiff('field', 2, 3)
         out = self.io.getoutput()
         self.assertTrue('field' in out)
 
     def test_showdiff_ints_no_color(self):
-        commands._showdiff('field', 2, 3, False)
+        config['color'] = False
+        commands._showdiff('field', 2, 3)
         out = self.io.getoutput()
         self.assertTrue('field' in out)
 
     def test_showdiff_shows_both(self):
-        commands._showdiff('field', 'old', 'new', True)
+        commands._showdiff('field', 'old', 'new')
         out = self.io.getoutput()
         self.assertTrue('old' in out)
         self.assertTrue('new' in out)
 
     def test_showdiff_floats_close_to_identical(self):
-        commands._showdiff('field', 1.999, 2.001, True)
+        commands._showdiff('field', 1.999, 2.001)
         out = self.io.getoutput()
         self.assertFalse('field' in out)
 
     def test_showdiff_floats_differenct(self):
-        commands._showdiff('field', 1.999, 4.001, True)
+        commands._showdiff('field', 1.999, 4.001)
         out = self.io.getoutput()
         self.assertTrue('field' in out)
 
     def test_showdiff_ints_colorizing_is_not_stringwise(self):
-        commands._showdiff('field', 222, 333, True)
+        commands._showdiff('field', 222, 333)
         complete_diff = self.io.getoutput().split()[1]
 
-        commands._showdiff('field', 222, 232, True)
+        commands._showdiff('field', 222, 232)
         partial_diff = self.io.getoutput().split()[1]
 
         self.assertEqual(complete_diff, partial_diff)
 
-AN_ID = "28e32c71-1450-463e-92bf-e0a46446fc11"
-class ManualIDTest(unittest.TestCase):
+class ShowChangeTest(_common.TestCase):
     def setUp(self):
-        _common.log.setLevel(logging.CRITICAL)
-        self.io = _common.DummyIO()
-        self.io.install()
-    def tearDown(self):
-        self.io.restore()
-
-    def test_id_accepted(self):
-        self.io.addinput(AN_ID)
-        out = commands.manual_id(False)
-        self.assertEqual(out, AN_ID)
-
-    def test_non_id_returns_none(self):
-        self.io.addinput("blah blah")
-        out = commands.manual_id(False)
-        self.assertEqual(out, None)
-
-    def test_url_finds_id(self):
-        self.io.addinput("http://musicbrainz.org/entity/%s?something" % AN_ID)
-        out = commands.manual_id(False)
-        self.assertEqual(out, AN_ID)
-
-class ShowChangeTest(unittest.TestCase):
-    def setUp(self):
-        self.io = _common.DummyIO()
+        super(ShowChangeTest, self).setUp()
         self.io.install()
 
         self.items = [_common.item()]
         self.items[0].track = 1
         self.items[0].path = '/path/to/file.mp3'
         self.info = autotag.AlbumInfo(
-            'the album', 'album id', 'the artist', 'artist id', [
-                autotag.TrackInfo('the title', 'track id', index=1)
+            u'the album', u'album id', u'the artist', u'artist id', [
+                autotag.TrackInfo(u'the title', u'track id', index=1)
         ])
 
-    def tearDown(self):
-        self.io.restore()
-
     def _show_change(self, items=None, info=None,
-                     cur_artist='the artist', cur_album='the album',
+                     cur_artist=u'the artist', cur_album=u'the album',
                      dist=0.1):
         items = items or self.items
         info = info or self.info
         mapping = dict(zip(items, info.tracks))
+        config['color'] = False
+        album_dist = distance(items, info, mapping)
+        album_dist._penalties = {'album': [dist]}
         commands.show_change(
             cur_artist,
             cur_album,
-            autotag.AlbumMatch(0.1, info, mapping, set(), set()),
-            color=False,
+            autotag.AlbumMatch(album_dist, info, mapping, set(), set()),
         )
         return self.io.getoutput().lower()
 
@@ -682,7 +626,7 @@ class ShowChangeTest(unittest.TestCase):
         self.assertTrue('correcting tags from:' in msg)
 
     def test_item_data_change(self):
-        self.items[0].title = 'different'
+        self.items[0].title = u'different'
         msg = self._show_change()
         self.assertTrue('different -> the title' in msg)
 
@@ -697,69 +641,27 @@ class ShowChangeTest(unittest.TestCase):
         self.assertTrue('correcting tags from:' in msg)
 
     def test_item_data_change_title_missing(self):
-        self.items[0].title = ''
-        msg = self._show_change()
+        self.items[0].title = u''
+        msg = re.sub(r'  +', ' ', self._show_change())
         self.assertTrue('file.mp3 -> the title' in msg)
 
     def test_item_data_change_title_missing_with_unicode_filename(self):
-        self.items[0].title = ''
+        self.items[0].title = u''
         self.items[0].path = u'/path/to/caf\xe9.mp3'.encode('utf8')
-        msg = self._show_change().decode('utf8')
+        msg = re.sub(r'  +', ' ', self._show_change().decode('utf8'))
         self.assertTrue(u'caf\xe9.mp3 -> the title' in msg
                         or u'caf.mp3 ->' in msg)
 
-class DefaultPathTest(unittest.TestCase):
-    def setUp(self):
-        self.old_home = os.environ.get('HOME')
-        self.old_appdata = os.environ.get('APPDATA')
-        os.environ['HOME'] = 'xhome'
-        os.environ['APPDATA'] = 'xappdata'
-    def tearDown(self):
-        if self.old_home is None:
-            del os.environ['HOME']
-        else:
-            os.environ['HOME'] = self.old_home
-        if self.old_appdata is None:
-            del os.environ['APPDATA']
-        else:
-            os.environ['APPDATA'] = self.old_appdata
-
-    def test_unix_paths_in_home(self):
-        import posixpath
-        config, lib, libdir = ui.default_paths(posixpath)
-        self.assertEqual(config, 'xhome/.beetsconfig')
-        self.assertEqual(lib, 'xhome/.beetsmusic.blb')
-        self.assertEqual(libdir, 'xhome/Music')
-
-    def test_windows_paths_in_home_and_appdata(self):
-        import ntpath
-        config, lib, libdir = ui.default_paths(ntpath)
-        self.assertEqual(config, 'xappdata\\beetsconfig.ini')
-        self.assertEqual(lib, 'xappdata\\beetsmusic.blb')
-        self.assertEqual(libdir, 'xhome\\Music')
-
-class PathFormatTest(unittest.TestCase):
-    def _config(self, text):
-        cp = ConfigParser.SafeConfigParser()
-        cp.readfp(StringIO(text))
-        return cp
-
-    def _paths_for(self, text):
-        return ui._get_path_formats(self._config("[paths]\n%s" %
-                                                 textwrap.dedent(text)))
-
-    def test_default_paths(self):
-        pf = self._paths_for("")
-        self.assertEqual(pf, ui.DEFAULT_PATH_FORMATS)
-
+class PathFormatTest(_common.TestCase):
     def test_custom_paths_prepend(self):
-        pf = self._paths_for("""
-            foo: bar
-        """)
+        default_formats = ui.get_path_formats()
+
+        config['paths'] = {u'foo': u'bar'}
+        pf = ui.get_path_formats()
         key, tmpl = pf[0]
         self.assertEqual(key, 'foo')
         self.assertEqual(tmpl.original, 'bar')
-        self.assertEqual(pf[1:], ui.DEFAULT_PATH_FORMATS)
+        self.assertEqual(pf[1:], default_formats)
 
 def suite():
     return unittest.TestLoader().loadTestsFromName(__name__)
